#!/bin/sh


#================================================================================
#
#   The following code figures out the path to the current script. The code
#   follows symbolic links, if they are present.
#
#================================================================================

found=0
progname=$0
curdir=`dirname "$progname"`
icdmagic="$curdir/__shell.sh"
if [ -x "$icdmagic" ]; then
    dist_bin=$curdir
    found=1
else
    here=`/bin/pwd`
    while [ -h "$progname" ]; do
        bn=`basename "$progname"`
        cd `dirname "$progname"`
        progname=`/bin/ls -l "$bn" |sed -e 's/^.* -> //' `
        if [ ! -x "$progname" ]; then
            # Not found.
            break
        fi
        curdir=`dirname "$progname"`
        icdmagic="$curdir/__shell.sh"
        if [ -x "$icdmagic" ]; then
            cd "$curdir"
            dist_bin=`pwd`
            found=1
            break
        fi
    done
    cd "$here"
fi

if [ $found = 0 ]; then
    echo "error: *** Installation problem."
    echo "error: *** Cannot find runtime directory."
    exit 1
fi


#================================================================================
#
#   Now we determine the architecture (os, os-version, and machine type).
#
#================================================================================

uname=`uname`
if [ $uname = "Linux" ]; then
    os=linux
    machine=`uname -m`
    osver=`uname -r | sed -e "s/\([0-9]\+[.][0-9]\+\)\(.*\)/\1/"`
elif [ $uname = "SunOS" ]; then
    os=sunos
    machine=`uname -m`
    osver=`uname -r`
elif [ $uname = "HP-UX" ]; then
    os=hpux
    machine=`uname -m | sed -e "s/[/]/./"`
    osver=`uname -r | sed -e "s/\([A-Z]\+[.]\)\(.*\)/\2/"`
else
    os=cygwin
    machine=`uname -m | sed -e "s/[/]/./"`
    osver=`uname -r | sed -e "s/\([0-9]\+[.][0-9]\+\).*/\1/"`
fi


#===============================================================================
#
#   Now we locate the tool to execute, using the architecture we just
#   determined. This tool is probably also a symbolic link (to the program
#   __shell), but we don't care at this point.
#
#===============================================================================

arch=${os}-${osver}-${machine}
tail=`basename $progname`

if [ ! -x $dist_bin/../../$arch/bin/$tail ]; then

    osmajor=`echo $osver | sed -e "s/[.][0-9.]\+//"`
    arch=${os}-${osmajor}.x-${machine}

    if [ ! -x $dist_bin/../../$arch/bin/$tail ]; then

        arch=${os}-x.x-${machine}

    if [ ! -x $dist_bin/../../$arch/bin/$tail ]; then
        arch=${os}-x.x-unknown
        if [ ! -x $dist_bin/../../$arch/bin/$tail ]; then

            echo "error: *** Tool \`$tail' not supported for architecture \`${os}-${osver}-${machine}'."
            #exit 1
        fi
    fi
    fi
fi

# Hack for running dali, simeye, etc. on other host
#
host=`echo $DISPLAY | sed -e "s/:.*//"`
if [ "$host" = "localhost" ]; then
host=`echo $DISPLAY | sed -e "s/.*://"`
DISPLAY=127.0.0.1:$host
fi

# Run the tool.
#
$dist_bin/../../$arch/bin/$tail $*

