
################################################################################
namespace eval std {
################################################################################

variable __std_architecture
catch {unset __std_architecture}

proc set_cpu {cpu} {
    variable __std_architecture
    catch {unset __std_architecture}
    set ::env(CACD_CPU) $cpu
}

proc get_architecture {{format long}} {

    variable __std_architecture
    if [info exists __std_architecture($format)] {
	return $__std_architecture($format)
    }

    if [info exists ::env(CACD_ARCH)] {
	if {$format == "long"} {
	    set __std_architecture($format) $::env(CACD_ARCH)
	} elseif {$format == "short"} {
	    if [info exists ::env(CACD_ARCH_SHORT)] {
		set __std_architecture($format) $::env(CACD_ARCH_SHORT)
	    } elseif ![regexp {([^-]*-[^-]*-[^-]*)-.*} $::env(CACD_ARCH) all __std_architecture($format)] {
		set __std_architecture($format) $::env(CACD_ARCH)
	    }
	}
	return $__std_architecture($format)
    }

    # Determine operating system
    #
    if {$::tcl_platform(platform) == "windows"} {
	set os cygwin
	set osver [exec uname -r]
    } else {
	set os [get_os]
	set osver $::tcl_platform(osVersion)
    }

    # Determine operating system version.
    #
    regsub {^[A-Z]*[.]} $osver "" osver
    regexp {^[0-9]*[.][0-9]*} $osver osver

    # Determine instruction set.
    #
    set iset ""
    if [info exists ::env(CACD_CPU)] {
        set iset $::env(CACD_CPU)
    }
    if ![string length $iset] {
	if {$::tcl_platform(platform) == "windows"} {
	    set iset [exec uname -m]
	} else {
	    set iset $::tcl_platform(machine)
	    regsub -all "/" $iset "." iset
	}
    }

    set suffix ""
    if {$format == "long"} {
        if [info exists ::env(HOSTNAME)] {
            set hostname $::env(HOSTNAME)
            regsub {[.].*} $hostname "" hostname
            if {$hostname == "charon"} {
                set suffix "-charon2"
            } elseif {$hostname == "charon" || $hostname == "io" || $hostname == "europa" || $hostname == "callisto" || $hostname == "metis" || $hostname == "himalia"} {
                set suffix "-$hostname"
            }
        }
    }

    set __std_architecture($format) "$os-$osver-$iset$suffix"
    return $__std_architecture($format)
}

proc get_os {} {
    if [info exists ::env(CACD_ARCH)] {
	regsub {[-].*} $::env(CACD_ARCH) "" os
    } else {
	set os $::tcl_platform(os)
	set os [string tolower $os]
	regsub -all {[- ]} $os "" os
    }
    return $os
}

proc get_shared_library_extension {} {
    set os [get_os]
    if {$os == "hpux"} {return ".sl"}
    return ".so"
}


################################################################################
}
################################################################################

