
std::include_once "src/generic/libs/tcl/std/obj.tcl"


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
namespace eval std {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

::obj::template {::object} {

    ::obj::method new {} {
    }

    ::obj::method to_string {} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set string "<${type} "
        set index -1
        foreach attr [::obj::attributes --dynamic ${type}] {
            incr index
            if {$index > 0} {append string ","}

            append string "$attr" 
            append string ":"
            if [::obj::is_assigned $obj $attr] {
                set value [::obj::get $obj $attr]
                set attr_type [::obj::attribute_info $type $attr get_type]
                if ![string equal [string index $attr_type 0] "%"] {
                    append string [::obj::invoke $value to_string]
                } else {
                    append string "$attr_type\[$value\]"
                }
            } else {
                append string "\[null\]"
            }
        }
        append string ">"
        return $string
    }

    ::obj::method dump {{indent 0} {fp stderr}} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\(${type}\)\n"
        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\{\n"

        foreach attr [lsort [::obj::attributes --dynamic -- $type]] {
            puts -nonewline $fp [std::mulstr " " [expr $indent+3]]
            puts -nonewline $fp "$attr=" 
            if [::obj::is_assigned $obj $attr] {
                set value [::obj::get $obj $attr]
                set attr_type [::obj::attribute_info $type $attr get_type]
                if ![string equal [string index $attr_type 0] "%"] {
                    puts -nonewline $fp "\n"
                    ::obj::invoke $value dump [expr $indent+6] $fp
                } else {
                    puts -nonewline $fp "$attr_type\[$value\]"
                    puts -nonewline $fp "\n"
                }
            } else {
                puts -nonewline $fp "\[null\]"
                puts -nonewline $fp "\n"
            }
        }

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\}\n"
    }

    ::obj::method #get_accessors {} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set list [list]
        foreach attr [::obj::attributes ${type}] {
            set attr_type [::obj::attribute_info $type $attr get_type]
            if [::obj::attribute_info $type $attr is_static] continue
            if ![::obj::is_assigned $obj $attr] continue
            if ![string equal [string index $attr_type 0] "%"] {
                lappend list $attr
            }
        }
        return $list
    }

    ::obj::method #get_all_accessors {} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set accessors [::obj::invoke $obj get_accessors]
        foreach accessor $accessors {
            set sub [::obj::invoke $obj #get $accessor]
            foreach elem [::obj::invoke $sub get_all_accessors] {
                lappend accessors [concat [list $accessor] $elem]
            }
        }
        return $accessors
    }

    ::obj::method #get {accessor} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set attr [lindex $accessor 0]
        set rest [lrange $accessor 1 end]

        set sub [::obj::get $obj $attr]
        if ![llength $rest] {
            return $sub
        }
        return [::obj::invoke $sub #get $rest]
    }

    ::obj::method #get_type {accessor} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set attr [lindex $accessor 0]
        set rest [lrange $accessor 1 end]

        if ![llength $rest] {
            return [::obj::attribute_info $this $attr get_type]
        }
        set sub [::obj::get $obj $attr]
        return [::obj::invoke $sub get_type $rest]
    }

    ::obj::method #set {accessor value} {

        set type [::obj::dynamic_type $this]
        set obj [::obj::dynamic_cast $this $type]

        set attr [lindex $accessor 0]
        set rest [lrange $accessor 1 end]

        if ![llength $rest] {
            ::obj::assign $obj $attr $value
            return
        }

        set sub [::obj::get $obj $attr]
        ::obj::invoke $sub #set $rest $value
    }

    ::obj::method normalize {} {
        # Do nothing.
    }
}

::obj::template {::string} {

    ::obj::inherit ::object

    ::obj::attribute rep %notype

    ::obj::method new {args} {
        if ![llength $args] {
            ::obj::assign $this rep ""
        } elseif {[llength $args] == 1} {
            ::obj::assign $this rep [lindex $args 0]
        } else {
            error "Invalid # arguments."
        }
    }

    ::obj::method to_string {} {
        return [::obj::get $this rep]
    }

    ::obj::method from_string {rep} {
        ::obj::assign $this rep $rep
    }

    ::obj::method get_value {} {
        return [::obj::get $this rep]
    }

    ::obj::method set_value {rep} {
        ::obj::assign $this rep $rep
    }
}

::obj::template {::double} {

    ::obj::inherit ::object

    ::obj::attribute rep %notype

    ::obj::method new {args} {
        if ![llength $args] {
            ::obj::assign $this rep 0.0
        } elseif {[llength $args] == 1} {
            set rep [expr double([lindex $args 0])]
            ::obj::assign $this rep $rep
        } else {
            error "Invalid # arguments."
        }
    }

    ::obj::method get_value {} {
        return [::obj::get $this rep]
    }

    ::obj::method set_value {rep} {
        set rep [expr double($rep)]
        ::obj::assign $this rep $rep
    }

    ::obj::method to_string {} {
        return [::obj::get $this rep]
    }

    ::obj::method from_string {rep} {
        set rep [expr double($rep)]
        ::obj::assign $this rep $rep
    }
}

::obj::template {::integer} {

    ::obj::inherit ::object

    ::obj::attribute rep %notype

    ::obj::method new {args} {
        if ![llength $args] {
            ::obj::assign $this rep 0
        } elseif {[llength $args] == 1} {
            set rep [expr int([lindex $args 0])]
            ::obj::assign $this rep $rep
        } else {
            error "Invalid # arguments."
        }
    }

    ::obj::method get_value {} {
        return [::obj::get $this rep]
    }

    ::obj::method set_value {rep} {
        set rep [expr int($rep)]
        ::obj::assign $this rep $rep
    }

    ::obj::method to_string {} {
        return [::obj::get $this rep]
    }

    ::obj::method from_string {rep} {
        set rep [expr int($rep)]
        ::obj::assign $this rep $rep
    }
}

::obj::template {::vector {type T}} {

    ::obj::inherit ::object

    ::obj::attribute rep %notype

    ::obj::method new {args} {
        ::obj::assign $this rep $args
    }

    ::obj::method clear {} {
        ::obj::assign $this rep [list]
    }

    ::obj::method push_back {value} {
        set list [::obj::get $this rep]
        lappend list $value
        ::obj::assign $this rep $list
    }

    ::obj::method remove_value {value} {
        set list [::obj::get $this rep]
        set index [lsearch -exact $list $value]
        # Note: if index < 0, then this also works.
        ::obj::assign $this rep [lreplace $list $index $index]
    }

    ::obj::method #get {accessor} {
        set index [lindex $accessor 0]
        set tail [lrange $accessor 1 end]
        set sub [lindex [::obj::get $this rep] $index]
        if ![llength $tail] {
            return $sub
        }
        return [::obj::invoke $sub #get $tail]
    }

    ::obj::method #set {accessor value} {
        set index [lindex $accessor 0]
        set tail [lrange $accessor 1 end]
        set rep [::obj::get $this rep]
        if ![llength $tail] {
            set rep [lreplace $rep $index $index $value]
            ::obj::assign $this rep $rep
        } else {
            set sub [lindex $rep $index]
            ::obj::invoke $sub #set $tail $value
        }
    }

    ::obj::method size {} {
        return [llength [::obj::get $this rep]]
    }

    ::obj::method #get_accessors {} {
        set size [llength [::obj::get $this rep]]
        set list [list]
        for {set i 0} {$i < $size} {incr i} {
            lappend list $i
        }
        return $list
    }

    ::obj::method get_values {} {
        return [::obj::get $this rep]
    }

    ::obj::method to_string {} {
        set string ""
        append string "{"
        set index -1
        foreach elem [::obj::get $this rep] {
            incr index
            if {$index > 0} {
                append string ","
            }
            append string [::obj::invoke $elem to_string]
        }
        append string "}"
        return $string
    }

    ::obj::method dump {{indent 0} {fp stderr}} {

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\([::obj::dynamic_type $this]\)\n"
        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\{\n"

        set index -1
        foreach elem [::obj::get $this rep] {
            incr index
            puts -nonewline $fp [std::mulstr " " [expr $indent+3]]
            puts -nonewline $fp "$index=" 
            puts -nonewline $fp "\n"
            ::obj::invoke $elem dump [expr $indent+6] $fp
        }

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\}\n"
    }

    ::obj::method normalize {} {
        set list [list]
        foreach elem [::obj::get $this rep] {
            ::obj::invoke $elem normalize
            lappend list $elem
        }
        ::obj::assign $this rep $list
    }
}

::obj::template {::string_string_map} {

    ::obj::inherit ::object

    ::obj::attribute rep %notype

    ::obj::method new {} {
        ::obj::assign $this rep [mapping new]
    }

    ::obj::method insert {key value} {
        set mapping [::obj::get $this rep]
        set mapping [mapping set $mapping $key $value]
        ::obj::assign $this rep $mapping
    }

    ::obj::method remove {key} {
        set mapping [::obj::get $this rep]
        set mapping [mapping unset $mapping $key]
        ::obj::assign $this rep $mapping
    }

    ::obj::method get {key} {
        return [mapping get [::obj::get $this rep] $key]
    }

    ::obj::method contains {key} {
        return [mapping contains [::obj::get $this rep] $key]
    }

    ::obj::method clear {} {
        ::obj::assign $this rep [mapping new]
    }

    ::obj::method size {} {
        return [llength [mapping names [::obj::get $this rep]]]
    }

    ::obj::method get_names {} {
        return [mapping names [::obj::get $this rep]]
    }

    ::obj::method to_string {} {
        set string ""
        append string "{"
        set index -1
        set mapping [::obj::get $this rep]
        foreach key [mapping names $mapping] {
            incr index
            if {$index > 0} {
                append string ","
            }
            set value [mapping get $mapping $key]
            append string "$key:$value"
        }
        append string "}"
        return $string
    }

    ::obj::method dump {{indent 0} {fp stderr}} {

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\([::obj::dynamic_type $this]\)\n"
        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\{\n"

        set index -1
        set mapping [::obj::get $this rep]
        foreach key [mapping names $mapping] {
            incr index
            puts -nonewline $fp [std::mulstr " " [expr $indent+3]]
            set value [mapping get $mapping $key]
            puts -nonewline $fp "$key=$value"
        }

        puts -nonewline $fp [std::mulstr " " $indent]
        puts -nonewline $fp "\}\n"
    }

    ::obj::method normalize {} {
        # Do nothing.
    }
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
}
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

