
################################################################################
namespace eval std {
################################################################################

################################################################################
#
#  Make sure that this file is included only once.
#
variable __only_once
if ![info exists __only_once] {
    set __only_once 1
################################################################################


#===============================================================================
#
#  Tcl settings.
#
#===============================================================================

# Full IEEE double precision; no loss of information.
#
set ::tcl_precision 17

# Make sure that random numbers come out the same each time. You can override 
# this, of course.
#
expr srand(1)


#===============================================================================
#
#  Normalize the CACD_DEVELOP_ROOT variable.
#
#===============================================================================

variable cacd_develop_root
variable root

namespace eval __tmp {
    global env

    # Note: pwd may fail if current working directory does not exist.
    #
    if [catch {set cwd [pwd]} msg] {
        puts stderr "warning: Current directory does not exist."
        set cwd $env(HOME)
    }

    catch {unset ::std::root}

    if [info exists env(CACD_DEVELOP_ROOT)] {
        if ![file isdirectory $env(CACD_DEVELOP_ROOT)] {
            puts stderr "warning: the environment variable `CACD_DEVELOP_ROOT' does not point to a valid directory."
            unset env(CACD_DEVELOP_ROOT)
        }
    }

    if [info exists env(CACD_DEVELOP_ROOT)] {
        if [catch {
            cd $env(CACD_DEVELOP_ROOT)
            set env(CACD_DEVELOP_ROOT) [pwd]
            set ::std::cacd_develop_root [pwd]
            set ::std::root $::std::cacd_develop_root
        }] {
            puts stderr "error: Cannot cd to \$CACD_DEVELOP_ROOT."
            exit 1
        }
    }

    if [info exists env(CACD_SCRIPT_ROOT)] {
        if ![file isdirectory $env(CACD_SCRIPT_ROOT)] {
            puts stderr "warning: the environment variable `CACD_SCRIPT_ROOT' does not point to a valid directory."
            unset env(CACD_SCRIPT_ROOT)
        }
    }

    if [info exists env(CACD_SCRIPT_ROOT)] {
        if [catch {
            cd $env(CACD_SCRIPT_ROOT)
            set env(CACD_SCRIPT_ROOT) [pwd]
            set ::std::root [pwd]
        }] {
            puts stderr "error: Cannot cd to \$CACD_SCRIPT_ROOT."
            exit 1
        }
    }

    if ![info exists ::std::root] {
        puts stderr "fatal: Please set your CACD_DEVELOP_ROOT or CACD_SCRIPT_ROOT environment variable."
        exit 1
    }

    cd $cwd
}
namespace delete __tmp


#===============================================================================
#
#  Make sure that we have the proper version of Tcl.
#
#===============================================================================

eval {
    global tcl_version
    set major [lindex [split $tcl_version "."] 0]
    set minor [lindex [split $tcl_version "."] 1]

    if {$major < 8 || ($major == 8 && $minor < 3)} {
        puts stderr "fatal: version of tclsh should be at least 8.3"
        puts stderr "fatal: (current version of [info nameofexecutable] is $tcl_version)"
        exit 1
    }
}


#===============================================================================
#
#  Set global variables.
#
#===============================================================================

eval {
    variable __included_table
    set __included_table($::std::root/src/generic/libs/tcl/std/std.tcl) 1
}


#===============================================================================
#
#  Make sure that the `cd' command also sets env(CWD).
#
#===============================================================================

rename ::cd ::__std_cd

proc ::cd {args} {
    set result [eval ::__std_cd $args]
    global env
    set ::env(CWD) [pwd]
    return $result
}


#===============================================================================
#
#  Miscellaneous procedures.
#
#===============================================================================

proc include_once {rel_file_path} {
    global env
    variable __included_table

    if ![info exists __included_table($::std::root/$rel_file_path)] {
        set __included_table($::std::root/$rel_file_path) 1
        uplevel 1 [list source $::std::root/$rel_file_path]
    }
}

proc not_implemented {msg} {
    error "error fatal: not implemented: $msg"
}

proc invariant {expr} {
    set cmd "global __std_invariant; set __std_invariant \[expr [list $expr]\]"
    uplevel 1 $cmd
    global __std_invariant
    if !$__std_invariant {
        unset __std_invariant
        error "error: invariant \"$expr\" failed"
    }
    unset __std_invariant
}


################################################################################
#
#  End of `make sure that this file is included only once'.
}
################################################################################

################################################################################
#
#  End of namespace eval.
}
################################################################################

