
################################################################################
namespace eval std {
################################################################################


#===============================================================================
#
#  topological_sort -- Topologically sort a dependency list.
#
#  The list (argument) should be of the form
#
#      { 
#          {master1 slave1 slave2 slave3} 
#          {master2 slave4 slave5 ... }
#      }
#
#  Masters need not be distinct (a master may appear several times). The 
#  slaves, of course, may also appear several times, also in the same sublist.
#
#===============================================================================

proc topological_sort {list} {

    catch {unset table}
    catch {unset table2}
    set masters [list]
    foreach tuple $list {
        set master [lindex $tuple 0]
        if ![info exists table($master)] {
            lappend masters $master
            set table($master) [list]
            set table(index,$master) -1
        }
        set tuple [lrange $tuple 1 end]
        foreach slave $tuple {
            if ![info exists table2($master,$slave)] {
                lappend table($master) $slave
                set table2($master,$slave) 1
            }
        }
    }
    set table(#counter) 0
    foreach master $masters {
        __topological_sort $master table
    }
    set list [list]
    for {set i $table(#counter)} {$i > 0} {incr i -1} {
        lappend list $table(at,$i)
    }
    return $list
}

proc __topological_sort {master _table} {
    upvar $_table table
    if {$table(index,$master) >= 0} return
    set table(index,$master) 0
    foreach slave $table($master) {
        __topological_sort $slave table
    }
    set table(index,$master) [incr table(#counter)]
    set table(at,$table(#counter)) $master
}

################################################################################
}
################################################################################

