
std::include_once src/generic/libs/tcl/std/path.tcl
std::include_once src/generic/libs/tcl/std/format.tcl
std::include_once src/generic/libs/tcl/std/classes.tcl
std::include_once src/generic/libs/tcl/std/exception.tcl
std::include_once src/generic/libs/tcl/std/array.tcl
std::include_once src/generic/libs/tcl/std/exec.tcl
std::include_once src/generic/libs/tcl/std/math.tcl
std::include_once src/generic/libs/tcl/std/file.tcl
std::include_once src/generic/libs/tcl/std/message.tcl
std::include_once src/generic/libs/tcl/std/classes.tcl


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
namespace eval project_tools {
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#===============================================================================
#
#  Determine technology description file (.s file).
#
#===============================================================================

proc determine_sfile {user_project_path} {

    set process_path [determine_process $user_project_path]

    set sfile $process_path/space.def.s
    if ![file exists $sfile] {
        std::throw user_exception "Unable to locate technology description (space.def.s) in process directory `$process_path'."
    }

    return $sfile
}


#===============================================================================
#
#  Determine maskdata file.
#
#===============================================================================

proc determine_maskdata_file {user_project_path} {

    set process_path [determine_process $user_project_path]

    set maskdata_file $process_path/maskdata
    if ![file exists $maskdata_file] {
        std::throw user_exception "Unable to locate file `maskdata' in process directory `$process_path'."
    }

    return $maskdata_file
}


#===============================================================================
#
#  Determine process.
#
#===============================================================================

proc determine_process {user_project_path} {

    if ![file exists $user_project_path/.dmrc] {
        std::throw user_exception "Directory `$user_project_path' is not a project."
    }

    std::auto_chdir $user_project_path {
        set fp [open "./.dmrc" r]
        gets $fp line
        gets $fp process_path
        close $fp

        if [regexp {^[ \t]*[0-9]+[ \t]*$} $process_path] {
            set process_number [expr int($process_path)]

            set fp [open $::env(ICDPATH)/share/lib/process/processlist r]
            set process_name ""
            while {[gets $fp line] >= 0} {
                regsub -all {#.*} $line "" line
                regsub -all {^[ \t]+} $line "" line
                regsub -all {[ \t]+$} $line "" line
                regsub -all {[ \t]+} $line " " line
                set list [split $line " "]
                if {[llength $list] != 2} continue
                set n [lindex $list 0]
                if {$n == $process_number} {
                    set process_name [lindex $list 1]
                    break
                }
            }
            close $fp

            if {$process_name == ""} {
                std::throw user_exception "Process number `$process_number' is invalid."
            }

            set process_path $::env(ICDPATH)/share/lib/process/$process_name
            if ![file isdirectory $process_path] {
                std::throw user_exception "Could not find process directory `$process_path' for process number `$process_number'."
            }
        }

        set process_path [std::normalize_path $process_path]
    }

    return $process_path
}


#===============================================================================
#
#  Determine lambda.
#
#===============================================================================

proc determine_lambda {user_project_path} {

    if ![file exists $user_project_path/.dmrc] {
        std::throw user_exception "Directory `$user_project_path' is not a project."
    }

    std::auto_chdir $user_project_path {
        set fp [open "./.dmrc" r]
        gets $fp line
        gets $fp line
        gets $fp lambda
        close $fp

        set lambda [std::validate_double $lambda 1]
    }

    return $lambda
}


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
}
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

